/***************************************************************************
                          i8ksensorslist.cpp  -  description
                             -------------------
    begin                : jue jun 20 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : migueln@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocalizedstring.h>

#include "i8ksensorslist.h"

#include "general.h"
#include "procinfo.h"


I8KSensorsList::I8KSensorsList( SensorDevsList *sensorDevsList, const QString &name ): SensorDev( sensorDevsList, name )
{
    struct Sensor::T_ValData valTemp, valFan;

    valTemp.init();
    valTemp.val = 30;
    valTemp.scale = Sensor::dgCelsius;

    valFan.init();
    valFan.val = 4500;

  setDescription(QString(i18n("MainBoard")) );

  setClass(Sensor::lmSensor);

  readConfig();

  if ( (i8kAvail = (I8KOnlyAvailable()==0)) )
  {
	cpuTemp= new Sensor( this );
	cpuTemp->setType(Sensor::lmTemp);
	cpuTemp->setObjectName( qsFL1( "cputemp" ) );
	cpuTemp->setDescription( qsFL1( "CPU Temp" ) );
	cpuTemp->setValueMax  (40 , Sensor::dgCelsius);
	cpuTemp->setValueMin  ( 0 , Sensor::dgCelsius);
	cpuTemp->setValueIdeal(30 , Sensor::dgCelsius);
	cpuTemp->setValue( valTemp );
	cpuTemp->readConfig();

	leftFan= new Sensor( this );
	leftFan->setType(Sensor::lmFan);
	leftFan->setObjectName( qsFL1( "fan1" ) );
	leftFan->setDescription( qsFL1( "Left Fan" ) );
	leftFan->setValueMax  (10000);
	leftFan->setValueMin  (    0);
	leftFan->setValueIdeal( 4500);
	leftFan->setValue( valFan );
	leftFan->readConfig();

	rightFan= new Sensor( this );
	rightFan->setType(Sensor::lmFan);
	rightFan->setObjectName( qsFL1( "fan2" ) );
	rightFan->setDescription( qsFL1( "Right Fan") );
	rightFan->setValueMax  (10000);
	rightFan->setValueMin  (    0);
	rightFan->setValueIdeal( 4500);
	rightFan->setValue( valFan );
	rightFan->readConfig();
  }

  if ( (acpiAvail=(AcpiAvailable()==0)) )
  {
	cpuTempAcpi= new Sensor( this );
	cpuTempAcpi->setType(Sensor::lmTemp);
	cpuTempAcpi->setObjectName( qsFL1( "cputemp" ) );
	cpuTempAcpi->setDescription( qsFL1( "CPU Temp" ) );
	cpuTempAcpi->setValueMax  (40 , Sensor::dgCelsius);
	cpuTempAcpi->setValueMin  ( 0 , Sensor::dgCelsius);
	cpuTempAcpi->setValueIdeal(30 , Sensor::dgCelsius);
	cpuTempAcpi->setValue( valTemp );
	cpuTempAcpi->readConfig();
  }

  updateSensors( false );
}

I8KSensorsList::~I8KSensorsList(){
}

void I8KSensorsList::updateSensors ( bool enabUpdate )
{
    struct Sensor::T_ValData valCpu, valLeftFan, valRightFan, valCpuAcpi;

    valCpu.init();
    valLeftFan.init(); 
    valRightFan.init();
    valCpuAcpi.init();

    valCpu.enabUpdate =  valLeftFan.enabUpdate = valRightFan.enabUpdate = valCpuAcpi.enabUpdate = enabUpdate;

    if( i8kAvail && !getI8KInfo( &valCpu.val, &valLeftFan.val, &valRightFan.val ) )
    {
	valCpu.scale = Sensor::dgCelsius;

	cpuTemp->setValue( valCpu );
	leftFan->setValue( valLeftFan );
	rightFan->setValue( valRightFan );
    }

    if( acpiAvail && !getAcpiTemperature( &valCpuAcpi.val ) )
    {
	valCpuAcpi.scale = Sensor::dgCelsius;

	cpuTempAcpi->setValue( valCpuAcpi );
    }

    setWithGlobalTimerUpdate ( false );
}


int I8KSensorsList::AcpiAvailable()
{
 double t;
 return  getAcpiTemperature(&t);
}

int I8KSensorsList::I8KOnlyAvailable()
{
double d1,d2,d3;

  return getI8KInfo(&d1,&d2,&d3);
}

bool I8KSensorsList::I8KAvailable()
{
int result;

 switch(result= I8KOnlyAvailable()) {
  case -2: qWarning("I18K Warning: /proc/i8k format not valid or not supported. Inform the author."); break;
  case -3: qWarning("I18K Warning: Only format version 1.0 is supported. Inform the author.");         break;
 }

 return (result==0 ||  AcpiAvailable()==0);
}



