/***************************************************************************
    file:           main.cpp
    description:    program main
    begin:          15/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qcommandlineparser.h>
#include <qdir.h>

#include <kaboutdata.h>
#include <klocalizedstring.h>
#include <kdbusservice.h>

#include "general.h"
#include "ksensorssplash.h"
#include "ksensorsmain.h"


static const char *description = "lmsensors and hddtemp frontend for KDE";

int main( int argc, char *argv[] )
{
    if (!qEnvironmentVariableIsSet ( "QT_QPA_PLATFORM" ))
        qputenv( "QT_QPA_PLATFORM", "xcb" );

    if (!qEnvironmentVariableIsSet ( "QT_LOGGING_RULES" ))
        qputenv( "QT_LOGGING_RULES", "*.debug=true;ks5.*=false;qt.*=false;kf.*=false" );


//  QString appPath = QApplication::applicationDirPath();
    QString appPath;
    QDir qdAppPath;

    qdAppPath = QDir();
    appPath = qdAppPath.absoluteFilePath ( qsFL1( argv[0] ) );

    qCDebug(qlcMain) << "\033[91m" << "main appPath" << appPath <<"\033[0m";

    int posFind = appPath.indexOf ( qsFL1( "/src" ) );

    if ( posFind >= 0)
    {
/*      QStringList qslPaths;
        QString qsPath, qsAppAppStd;

        qsAppPathStd = appPath.left (posFind);
        qslPaths = QStandardPaths::standardLocations ( QStandardPaths::GenericDataLocation );
        qslPaths.append (qsAppPathStd);

        qsPath = "";
        for (QStringList::const_iterator dir = qslPaths.constBegin(); dir != qslPaths.constEnd(); ++dir)
        {
            if (!qsPath.isEmpty())
                qsPath.append ( ":" );
              qsPath.append ( *dir );
        }

        qCDebug(qlcMain) << "\033[91m" << "main qsPath" << qsPath <<"\033[0m"; */


        QString qsPath, qsAppPathStd;

        qsAppPathStd = appPath.left ( posFind );

        qsPath = qsFL1( qgetenv ( "XDG_DATA_DIRS" ) );
        if (!qsPath.isEmpty())
            qsPath.append ( qsFL1( ":" ) );

        qsPath.append ( qsAppPathStd );

        qputenv ( "XDG_DATA_DIRS", qsPath.toUtf8() );

//        qDebug() << "\033[91m" << "qDebug(): main qsPath" << qsPath <<"\033[0m";
        
    } // if ( posFind >= 0 )

  
  QApplication app (argc, argv);
  
  KLocalizedString::setApplicationDomain( COMPNAME );
  
  KAboutData aboutData( QStringLiteral ( COMPNAME ), i18n("KSensors5"),
    QStringLiteral ( COMPVERS ), i18n (description), KAboutLicense::GPL_V2,
    ki18n ("(c) 2023, Giuseppe Della Bianca").toString(), QString(), qsFL1( "https://sourceforge.net/projects/ksensors5/" ), QStringLiteral ("giusdbg@gmail.com"));

  aboutData.addAuthor( i18n ("Giuseppe Della Bianca"), i18n("Author"), qsFL1( "giusdbg@gmail.com" ) );
  aboutData.addAuthor( i18n ("original author Miguel Novas"), QString(), qsFL1( "michaell@teleline.es" ) );
  aboutData.setOrganizationDomain( QByteArray( /* "kde.org" */ COMPDOM) );
//  aboutData.setProductName(QByteArray( COMPNAME ));      
  KAboutData::setApplicationData( aboutData );
     

  KDBusService service(KDBusService::Unique);
// service.connect(&service, &KDBusService::activateRequested, &app, &KGpgApp::slotDBusActivation);
  
  
  QCommandLineParser parser;
  KAboutData::applicationData().setupCommandLine( &parser );  
  parser.setApplicationDescription( i18n (description) );
//  parser.addHelpOption();
//  parser.addVersionOption();  
    
  QCommandLineOption autostartOpt( QStringList{ qsFL1( "autostart" ) }, i18n( "Only start if autostart is enabled." ));
  QCommandLineOption nodockOpt( QStringList{ qsFL1( "nodock" ) }, i18n( "Don't dock in KDE system tray." ));
  QCommandLineOption splashOpt( QStringList{ qsFL1( "splash" ) }, i18n( "Show splash screen." ));
  QCommandLineOption chartTestOpt( QStringList{ qsFL1( "charttest" ) }, i18n( "Show chart test window" ));
//  QCommandLineOption noforkOpt( QStringList{"nofork"}, i18n("none."));
  parser.addOption( autostartOpt );
  parser.addOption( nodockOpt );
  parser.addOption( splashOpt );
  parser.addOption( chartTestOpt );
//  parser.addOption( noforkOpt );
  
  parser.process (app);
  
  bool isAutoStart = parser.isSet( autostartOpt );
  bool noDock = parser.isSet( nodockOpt );
  bool showSplash =  parser.isSet ( splashOpt );

  
  app.setWindowIcon(QIcon::fromTheme( qsFL1( COMPICON ) ) );
  
  
  KConfigGroup cfg = KSharedConfig::openConfig()->group( qsFL1( "General" ) );
  
  if (isAutoStart && !cfg.readEntry( qsFL1( "AutoStart" ), true)) return 0;
  if( cfg.readEntry("Version")!=aboutData.version() ) {
    cfg.writeEntry("Version",aboutData.version());
    showSplash= true;
  }

  if(showSplash)  (void)new KSensorsSplash();

    if ( !QSystemTrayIcon::isSystemTrayAvailable() )
        noDock = true;
  
    KsensorsMain *ksMain= new KsensorsMain();
  
    ksMain->setPar ( parser );

    ksMain->setPar ( qsFL1( "isAutoStartOpt" ), isAutoStart );
    ksMain->setPar ( qsFL1( "noDockOpt" ), noDock );
    ksMain->setPar ( qsFL1( "showSplashOpt" ), showSplash );

    ksMain->init();


  QObject::connect( ksMain, SIGNAL(destroyed()), &app, SLOT(quit()) );
//  QObject::connect( &app, &QApplication::aboutToQuit, ksensors, [=]() { ksensors->saveConfig ( false ); } ); 
  QObject::connect( qApp, &QGuiApplication::commitDataRequest, ksMain, [=]() { ksMain->saveConfig ( false ); } );
  
  app.setQuitOnLastWindowClosed(false);
  
  return app.exec();
}
