/***************************************************************************
                          sensor.cpp  -  description
                             -------------------
    begin                : mi� abr 24 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdebug.h>

#include "general.h"
#include "sensordevslist.h"
#include "sensordev.h"
#include "sensor.h"


Sensor::Sensor( SensorDev *parent, const QString &name): QObject( /* parent */ /* ,name */ )
{
  setObjectName ( name );
  
  valSensors.init();
  valDescr.clear();
  descr1.clear(); 
  descr2.clear();

  valPrevious = compensation = valMax = valMin = valIdeal = 0.0;
  multiplicator = 1.0;
  
  description.clear();
  monitorize   = false;
  
  setParent( parent );
}


Sensor::~Sensor(){
}

/***************************************************************************
    get/set names e description, object name of sensor device
 ***************************************************************************/

void Sensor::setShortName( const QString &name )
{
    shortName = name;
}

QString Sensor::getShortName()
{
    return shortName;
}
    
void Sensor::setLongName( const QString &name )
{
    longName = name;
}

QString Sensor::getLongName()
{
    return longName;
}

void Sensor::setSubName( const QString &name )
{
    subName = name;
}

QString Sensor::getSubName()
{
    return subName;
}

void Sensor::setDescription( const QString &desc )
{
    description = desc;
}

const QString &Sensor::getDescription()
{
    return description;
}

void Sensor::setObjectName( const QString &name )
{
    QObject::setObjectName( name );
    
    setShortName( name );
    setLongName( name );
}

struct Sensor::stValMask Sensor::getPrintMask(bool addSufix)
{        
    return getSensorPrintMask(type, addSufix, ((SensorDev*)parent())->getTempScale() );
}

struct Sensor::stValMask Sensor::getSensorPrintMask(int sensorType, bool addSufix, TempScale temp)
{
    struct stValMask valMask;
      
    valMask.init();
    valMask.width = 0;
    valMask.prec = 0;
    valMask.formType = 'f';
    valMask.postTxt = qsFL1( "" );
    
 if(addSufix) {
    switch(sensorType) 
    {
      case Sensor::lmTemp:
         switch(temp) 
         {
           case dgCelsius:
              valMask.postTxt = qsFL1( " C" );
             
              break;
           case dgFahrenheit:
              valMask.postTxt = qsFL1( " F" );
             
              break;
           case dgKelvin:
              valMask.postTxt = qsFL1( " K" );
           
              break;
        default:             
           break;
         }
         break;
      case Sensor::lmFan:
         valMask.postTxt = qsFL1( " rpm" );
        
         break;
      case Sensor::lmVoltage:
         valMask.prec = 2;
         valMask.postTxt = qsFL1( "V" );
         
         break;
      default:
        
        break;
    }
 }
 
    if ( valMask.postTxt.isEmpty() )
        valMask.prec = ( sensorType == Sensor::lmVoltage ? 2 : 0 );
 
    return valMask;
}

bool Sensor::monitorized()
{
  return monitorize;
}

void Sensor::setMonitorized(bool enable)
{
 if( monitorize!=enable ) monitorize= enable;
}

void Sensor::setValueIdeal(double value, TempScale scale)
{
 valIdeal= toCelsius(value, scale);
}
	
void Sensor::setValueMax(double value, TempScale scale)
{
  valMax= toCelsius(value,scale);
}

void Sensor::setValueMin(double value, TempScale scale)
{
  valMin= toCelsius(value,scale);
}

void Sensor::setCompensation(double value, TempScale scale)
{
  compensation= toCelsiusDiff(value,scale);
}

void Sensor::setMultiplicator(double value)
{
 multiplicator= value;
}

void Sensor::setValue ( struct T_ValData &valData )
{  
    valData.val= toCelsius( valData.val, valData.scale );
    if ( valData.adjust ) valData.val = adjustValue( valData.val );

    valSensors.valueChanged = valSensors.val != valData.val || valSensors.val1 != valData.val1 || valSensors.val2 != valData.val2 || valSensors.val3 != valData.val3 || valSensors.val4 != valData.val4  || valData.enabUpdate;

    valPrevious = valSensors.val;
    valSensors.val = valData.val;
    valSensors.val1 = valData.val1;
    valSensors.val2 = valData.val2;
    valSensors.val3 = valData.val3;
    valSensors.val4 = valData.val4;

// qDebug() << "\033[91m" << "qDebug(): " << QDateTime::currentDateTime() << " Sensor::setValue()" << "name " << getLongName() << "\033[0m";
    
    ((SensorDev*)parent())->getSensorDevsList()->getKsensorsMain()->updateValue ( getLongName() );
    
    if ( valSensors.valueChanged ) 
    {   
        Q_EMIT valueChanged();
    }
}

void Sensor::writeConfig()
{
//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );
    setEntryConf ( qsFL1 ( "description" )     ,description);
    setEntryConf ( qsFL1 ( "valueIdeal" )      ,valIdeal);
    setEntryConf ( qsFL1 ( "valueMax" )        ,valMax);
    setEntryConf ( qsFL1 ( "valueMin" )        ,valMin);
    setEntryConf ( qsFL1 ( "compensation" )    ,compensation);
    setEntryConf ( qsFL1 ( "multiplicator" )   ,multiplicator);
    setEntryConf ( qsFL1 ( "monitorize" )      ,monitorize);
}

void Sensor::readConfig()
{
//  setGroupConf ( objectName() );
    setSensorDevGroupConf ( getShortName(), getLongName(), getSubName() );

    valMax = getEntryConf ( qsFL1 ( "valueMax" ), valMax, ksensorsSetting::Default );
    valMin = getEntryConf ( qsFL1 ( "valueMin" ), valMin, ksensorsSetting::Default );
    compensation = getEntryConf ( qsFL1 ( "compensation" ) ,compensation, ksensorsSetting::Default );
    multiplicator= getEntryConf ( qsFL1 ( "multiplicator" ), multiplicator, ksensorsSetting::Default );
    description = getEntryConf ( qsFL1 ( "description" ), description, ksensorsSetting::Default );
    valIdeal = getEntryConf ( qsFL1 ( "valueIdeal" ), valIdeal, ksensorsSetting::Default );
    setMonitorized ( getEntryConf ( qsFL1 ( "monitorize" ), monitorize, ksensorsSetting::Default ) );
    
//    qDebug() << "\033[91m" << "qDebug(): Sensor::readConfig " << objectName() << " valMax " << valMax <<"\033[0m";  
}

double Sensor::toCelsius(double val, TempScale scale)
{
  if(type!=lmTemp) return val;
  switch( scale == dgDefault ? getSensorDev()->getTempScale(): scale ) {
   case dgCelsius   : return val;
   case dgFahrenheit: return  (5.0/9.0)*(val-32.0);
  default:
    
    break;
  }
  return  val - 273.16;  // Kelvin to Celsius
}

double Sensor::celsiusTo(double val, TempScale scale)
{
  if(type!=lmTemp) return val;
  switch( scale==dgDefault ? getSensorDev()->getTempScale() : scale ) {
   case dgCelsius   : return val;
   case dgFahrenheit: return  (9.0/5.0)*val+32.0;
  default:
    
    break;
  }
  return  val + 273.16;  // Celsius to Kelvin
}

double Sensor::toCelsiusDiff(double val, TempScale scale)
{
  if(scale==dgDefault) scale= getSensorDev()->getTempScale();
  if(type!=lmTemp || scale!=dgFahrenheit ) return val;
  return val*(5.0/9.0);
}

double Sensor::celsiusToDiff(double val, TempScale scale)
{
  if(scale==dgDefault) scale= getSensorDev()->getTempScale();
  if(type!=lmTemp || scale!=dgFahrenheit ) return val;
  return val*(9.0/5.0);
}
