/***************************************************************************
                          sensorDevsList.h  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SENSORDEVSLIST_H
#define _SENSORDEVSLIST_H

#include <qobject.h>
// #include <qobjcoll.h>
#include <qstringlist.h>

#include <sensors/sensors.h>

#include "ksensorsmain.h"
#include "lmsensorschip.h"
#include "hdsensorslist.h"
#include "i8ksensorslist.h"
#include "syssensorslist.h"

class KsensorsMain;

class SensorDevsList : public QObject
{
Q_OBJECT

  friend class LMSensor;

public:

    SensorDevsList( KsensorsMain *ksensorsMainIn, QObject *parent = nullptr, const QString &name = QString() );
    ~SensorDevsList();

  void setMonitorized(bool enable);
  const QObjectList   *getSensorsChips();
  SensorDev *getSensorsChip(int index) { return !children().isEmpty() ? (SensorDev *)children().at(index) : nullptr; }
  SensorDev   *getSensorsChip( const QString &name ) { return findChild<SensorDev *>(name); }
  int count() 					  { return !children().isEmpty() ? children().count() : 0; }
  Sensor *getSensor( const QString &name );
  
    KsensorsMain *getKsensorsMain() { return ( ksensorsMain ); };

  QMultiMap<QString, SensorDev *> qmmSensorDevsList;
//  QMultiMap<QString, PanelsWindow *> qmPanelsWindow;
  
Q_SIGNALS:

  void valueChanged(Sensor *);

private:

  bool initSensors();
  void createLMSensors();
  void createHDSensors();
  void createI8KSensors();
  void createSysSensors();
  void childEvent ( QChildEvent *e ) override;

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
  static int existSensor(const sensors_chip_name *chip_name,const char *sensor_name);
#endif
  
    KsensorsMain *ksensorsMain;
};

#endif // _SENSORDEVSLIST_H
